@@ -11,6 +11,7 @@ only specify one user-data per instance.
 import pytest
 import yaml
 
+from tests.integration_tests.clouds import ImageSpecification
 from tests.integration_tests.decorators import retry
 from tests.integration_tests.util import get_console_log
 
@@ -64,6 +65,23 @@ chpasswd:
 """
 )
 
+USERS_USER_DATA = (
+    COMMON_USER_DATA
+    + """
+chpasswd:
+  users:
+    - name: tom
+      password: mypassword123!
+      type: text
+    - name: dick
+      type: RANDOM
+    - name: harry
+      type: RANDOM
+    - name: mikey
+      password: $5$xZ$B2YGGEx2AOf4PeW48KC6.QyT1W2B4rZ9Qbltudtha89
+"""
+)
+
 USERS_DICTS = yaml.safe_load(COMMON_USER_DATA)["users"]
 USERS_PASSWD_VALUES = {
     user_dict["name"]: user_dict["passwd"]
@@ -160,22 +178,36 @@ class Mixin:
         shadow = class_client.read_from_file("/etc/shadow")
         for user_dict in USERS_DICTS:
             if "name" in user_dict:
-                assert "{}:".format(user_dict["name"]) in shadow
+                assert f'{user_dict["name"]}:' in shadow
 
-    def test_sshd_config(self, class_client):
-        """Test that SSH password auth is enabled."""
-        sshd_config = class_client.read_from_file("/etc/ssh/sshd_config")
+    def test_sshd_config_file(self, class_client):
+        """Test that SSH config is written in the correct file."""
+        if ImageSpecification.from_os_image().release in {"bionic"}:
+            sshd_file_target = "/etc/ssh/sshd_config"
+        else:
+            sshd_file_target = "/etc/ssh/sshd_config.d/50-cloud-init.conf"
+        assert class_client.execute(f"ls {sshd_file_target}").ok
+        sshd_config = class_client.read_from_file(sshd_file_target)
         # We look for the exact line match, to avoid a commented line matching
         assert "PasswordAuthentication yes" in sshd_config.splitlines()
 
+    def test_sshd_config(self, class_client):
+        """Test that SSH password auth is enabled."""
+        sshd_config = class_client.execute("sshd -T").stdout
+        assert "passwordauthentication yes" in sshd_config
+
 
-@pytest.mark.ci
 @pytest.mark.user_data(LIST_USER_DATA)
 class TestPasswordList(Mixin):
     """Launch an instance with LIST_USER_DATA, ensure Mixin tests pass."""
 
 
-@pytest.mark.ci
 @pytest.mark.user_data(STRING_USER_DATA)
 class TestPasswordListString(Mixin):
     """Launch an instance with STRING_USER_DATA, ensure Mixin tests pass."""
+
+
+@pytest.mark.ci
+@pytest.mark.user_data(USERS_USER_DATA)
+class TestPasswordUsersList(Mixin):
+    """Launch an instance with USERS_USER_DATA, ensure Mixin tests pass."""
