@@ -3,6 +3,7 @@ import logging
 import textwrap
 
 import pytest
+import responses
 
 from cloudinit import util
 from cloudinit.config import cc_puppet
@@ -11,17 +12,19 @@ from cloudinit.config.schema import (
     get_schema,
     validate_cloudconfig_schema,
 )
-from tests.unittests.helpers import (
-    CiTestCase,
-    HttprettyTestCase,
-    mock,
-    skipUnlessJsonSchema,
-)
+from tests.unittests.helpers import CiTestCase, mock, skipUnlessJsonSchema
 from tests.unittests.util import get_cloud
 
 LOG = logging.getLogger(__name__)
 
 
+@pytest.fixture
+def fake_tempdir(mocker, tmpdir):
+    mocker.patch(
+        "cloudinit.config.cc_puppet.temp_utils.tempdir"
+    ).return_value.__enter__.return_value = str(tmpdir)
+
+
 @mock.patch("cloudinit.config.cc_puppet.subp.which")
 @mock.patch("cloudinit.config.cc_puppet.subp.subp")
 @mock.patch("cloudinit.config.cc_puppet.os")
@@ -143,11 +146,13 @@ class TestPuppetHandle(CiTestCase):
     def test_puppet_config_installs_puppet_aio(self, m_subp, m_aio, _):
         """Cloud-config with 'puppet' key installs
         when 'install_type' is 'aio'."""
-
-        self.cloud.distro = mock.MagicMock()
+        distro = mock.MagicMock()
+        self.cloud.distro = distro
         cfg = {"puppet": {"install": True, "install_type": "aio"}}
         cc_puppet.handle("notimportant", cfg, self.cloud, LOG, None)
-        m_aio.assert_called_with(cc_puppet.AIO_INSTALL_URL, None, None, True)
+        m_aio.assert_called_with(
+            distro, cc_puppet.AIO_INSTALL_URL, None, None, True
+        )
 
     @mock.patch("cloudinit.config.cc_puppet.install_puppet_aio", autospec=True)
     @mock.patch("cloudinit.config.cc_puppet.subp.subp", return_value=("", ""))
@@ -156,8 +161,8 @@ class TestPuppetHandle(CiTestCase):
     ):
         """Cloud-config with 'puppet' key installs
         when 'install_type' is 'aio' and 'version' is specified."""
-
-        self.cloud.distro = mock.MagicMock()
+        distro = mock.MagicMock()
+        self.cloud.distro = distro
         cfg = {
             "puppet": {
                 "install": True,
@@ -167,7 +172,7 @@ class TestPuppetHandle(CiTestCase):
         }
         cc_puppet.handle("notimportant", cfg, self.cloud, LOG, None)
         m_aio.assert_called_with(
-            cc_puppet.AIO_INSTALL_URL, "6.24.0", None, True
+            distro, cc_puppet.AIO_INSTALL_URL, "6.24.0", None, True
         )
 
     @mock.patch("cloudinit.config.cc_puppet.install_puppet_aio", autospec=True)
@@ -177,8 +182,8 @@ class TestPuppetHandle(CiTestCase):
     ):
         """Cloud-config with 'puppet' key installs
         when 'install_type' is 'aio' and 'collection' is specified."""
-
-        self.cloud.distro = mock.MagicMock()
+        distro = mock.MagicMock()
+        self.cloud.distro = distro
         cfg = {
             "puppet": {
                 "install": True,
@@ -188,7 +193,7 @@ class TestPuppetHandle(CiTestCase):
         }
         cc_puppet.handle("notimportant", cfg, self.cloud, LOG, None)
         m_aio.assert_called_with(
-            cc_puppet.AIO_INSTALL_URL, None, "puppet6", True
+            distro, cc_puppet.AIO_INSTALL_URL, None, "puppet6", True
         )
 
     @mock.patch("cloudinit.config.cc_puppet.install_puppet_aio", autospec=True)
@@ -198,8 +203,8 @@ class TestPuppetHandle(CiTestCase):
     ):
         """Cloud-config with 'puppet' key installs
         when 'install_type' is 'aio' and 'aio_install_url' is specified."""
-
-        self.cloud.distro = mock.MagicMock()
+        distro = mock.MagicMock()
+        self.cloud.distro = distro
         cfg = {
             "puppet": {
                 "install": True,
@@ -209,7 +214,7 @@ class TestPuppetHandle(CiTestCase):
         }
         cc_puppet.handle("notimportant", cfg, self.cloud, LOG, None)
         m_aio.assert_called_with(
-            "http://test.url/path/to/script.sh", None, None, True
+            distro, "http://test.url/path/to/script.sh", None, None, True
         )
 
     @mock.patch("cloudinit.config.cc_puppet.install_puppet_aio", autospec=True)
@@ -219,8 +224,8 @@ class TestPuppetHandle(CiTestCase):
     ):
         """Cloud-config with 'puppet' key installs
         when 'install_type' is 'aio' and no cleanup."""
-
-        self.cloud.distro = mock.MagicMock()
+        distro = mock.MagicMock()
+        self.cloud.distro = distro
         cfg = {
             "puppet": {
                 "install": True,
@@ -229,7 +234,9 @@ class TestPuppetHandle(CiTestCase):
             }
         }
         cc_puppet.handle("notimportant", cfg, self.cloud, LOG, None)
-        m_aio.assert_called_with(cc_puppet.AIO_INSTALL_URL, None, None, False)
+        m_aio.assert_called_with(
+            distro, cc_puppet.AIO_INSTALL_URL, None, None, False
+        )
 
     @mock.patch("cloudinit.config.cc_puppet.subp.subp", return_value=("", ""))
     def test_puppet_config_installs_puppet_version(self, m_subp, _):
@@ -402,68 +409,82 @@ URL_MOCK = mock.Mock()
 URL_MOCK.contents = b'#!/bin/bash\necho "Hi Mom"'
 
 
+@pytest.mark.usefixtures("fake_tempdir")
 @mock.patch("cloudinit.config.cc_puppet.subp.subp", return_value=(None, None))
 @mock.patch(
     "cloudinit.config.cc_puppet.url_helper.readurl",
     return_value=URL_MOCK,
     autospec=True,
 )
-class TestInstallPuppetAio(HttprettyTestCase):
-    def test_install_with_default_arguments(self, m_readurl, m_subp):
-        """Install AIO with no arguments"""
-        cc_puppet.install_puppet_aio()
-
-        self.assertEqual(
-            [mock.call([mock.ANY, "--cleanup"], capture=False)],
-            m_subp.call_args_list,
-        )
-
-    def test_install_with_custom_url(self, m_readurl, m_subp):
-        """Install AIO from custom URL"""
-        cc_puppet.install_puppet_aio("http://custom.url/path/to/script.sh")
-        m_readurl.assert_called_with(
-            url="http://custom.url/path/to/script.sh", retries=5
-        )
-
-        self.assertEqual(
-            [mock.call([mock.ANY, "--cleanup"], capture=False)],
-            m_subp.call_args_list,
-        )
-
-    def test_install_with_version(self, m_readurl, m_subp):
-        """Install AIO with specific version"""
-        cc_puppet.install_puppet_aio(cc_puppet.AIO_INSTALL_URL, "7.6.0")
-
-        self.assertEqual(
-            [mock.call([mock.ANY, "-v", "7.6.0", "--cleanup"], capture=False)],
-            m_subp.call_args_list,
-        )
-
-    def test_install_with_collection(self, m_readurl, m_subp):
-        """Install AIO with specific collection"""
-        cc_puppet.install_puppet_aio(
-            cc_puppet.AIO_INSTALL_URL, None, "puppet6-nightly"
-        )
-
-        self.assertEqual(
-            [
-                mock.call(
-                    [mock.ANY, "-c", "puppet6-nightly", "--cleanup"],
-                    capture=False,
-                )
-            ],
-            m_subp.call_args_list,
-        )
-
-    def test_install_with_no_cleanup(self, m_readurl, m_subp):
-        """Install AIO with no cleanup"""
-        cc_puppet.install_puppet_aio(
-            cc_puppet.AIO_INSTALL_URL, None, None, False
-        )
-
-        self.assertEqual(
-            [mock.call([mock.ANY], capture=False)], m_subp.call_args_list
-        )
+class TestInstallPuppetAio:
+    @pytest.mark.parametrize(
+        "args, expected_subp_call_args_list, expected_readurl_call_args_list",
+        [
+            pytest.param(
+                [],
+                [mock.call([mock.ANY, "--cleanup"], capture=False)],
+                [
+                    mock.call(
+                        url="https://raw.githubusercontent.com/puppetlabs/install-puppet/main/install.sh",  # noqa: 501
+                        retries=5,
+                    )
+                ],
+                id="default_arguments",
+            ),
+            pytest.param(
+                ["http://custom.url/path/to/script.sh"],
+                [mock.call([mock.ANY, "--cleanup"], capture=False)],
+                [
+                    mock.call(
+                        url="http://custom.url/path/to/script.sh", retries=5
+                    )
+                ],
+                id="custom_url",
+            ),
+            pytest.param(
+                [cc_puppet.AIO_INSTALL_URL, "7.6.0"],
+                [
+                    mock.call(
+                        [mock.ANY, "-v", "7.6.0", "--cleanup"], capture=False
+                    )
+                ],
+                [mock.call(url=cc_puppet.AIO_INSTALL_URL, retries=5)],
+                id="version",
+            ),
+            pytest.param(
+                [cc_puppet.AIO_INSTALL_URL, None, "puppet6-nightly"],
+                [
+                    mock.call(
+                        [mock.ANY, "-c", "puppet6-nightly", "--cleanup"],
+                        capture=False,
+                    )
+                ],
+                [mock.call(url=cc_puppet.AIO_INSTALL_URL, retries=5)],
+                id="collection",
+            ),
+            pytest.param(
+                [cc_puppet.AIO_INSTALL_URL, None, None, False],
+                [mock.call([mock.ANY], capture=False)],
+                [mock.call(url=cc_puppet.AIO_INSTALL_URL, retries=5)],
+                id="no_cleanup",
+            ),
+        ],
+    )
+    @responses.activate
+    def test_install_puppet_aio(
+        self,
+        m_readurl,
+        m_subp,
+        args,
+        expected_subp_call_args_list,
+        expected_readurl_call_args_list,
+        tmpdir,
+    ):
+        distro = mock.Mock()
+        distro.get_tmp_exec_path.return_value = str(tmpdir)
+        cc_puppet.install_puppet_aio(distro, *args)
+        assert expected_readurl_call_args_list == m_readurl.call_args_list
+        assert expected_subp_call_args_list == m_subp.call_args_list
 
 
 class TestPuppetSchema:
